@@ -253,9 +253,25 @@ def write_trap(method, obj_cls):
             raise StateModifiedError()
         args = tuple(proxy(a) for a in args)
         kwargs = {k: proxy(v) for k, v in kwargs.items()}
+        old = self.target.copy()
         retval = fn(self.target, *args, **kwargs)
-        # TODO: prevent firing if value hasn't actually changed?
-        proxy_db.attrs(self)["dep"].notify()
+        attrs = proxy_db.attrs(self)
+        if obj_cls == dict:
+            change_detected = False
+            keydeps = attrs["keydep"]
+            for key, val in self.target.items():
+                if old.get(key) is not val:
+                    if key in keydeps:
+                        keydeps[key].notify()
+                    else:
+                        keydeps[key] = Dep()
+                    change_detected = True
+            if change_detected:
+                attrs["dep"].notify()
+        else:  # list and set
+            if self.target != old:
+                attrs["dep"].notify()
+
         return retval
 
     return inner
@@ -270,17 +286,18 @@ def write_key_trap(method, obj_cls):
         if Dep.stack:
             raise StateModifiedError()
         key = args[0]
-        is_new = key not in proxy_db.attrs(self)["keydep"]
+        attrs = proxy_db.attrs(self)
+        is_new = key not in attrs["keydep"]
         old_value = getitem_fn(self.target, key) if not is_new else None
         args = [key] + [proxy(a) for a in args[1:]]
         kwargs = {k: proxy(v) for k, v in kwargs.items()}
         retval = fn(self.target, *args, **kwargs)
         new_value = getitem_fn(self.target, key)
         if is_new:
-            proxy_db.attrs(self)["keydep"][key] = Dep()
+            attrs["keydep"][key] = Dep()
         if old_value != new_value:
-            proxy_db.attrs(self)["keydep"][key].notify()
-            proxy_db.attrs(self)["dep"].notify()
+            attrs["keydep"][key].notify()
+            attrs["dep"].notify()
         return retval
 
     return inner
@@ -294,10 +311,11 @@ def delete_trap(method, obj_cls):
         if Dep.stack:
             raise StateModifiedError()
         retval = fn(self.target, *args, **kwargs)
-        proxy_db.attrs(self)["dep"].notify()
+        attrs = proxy_db.attrs(self)
+        attrs["dep"].notify()
         for key in self._orphaned_keydeps():
-            proxy_db.attrs(self)["keydep"][key].notify()
-            del proxy_db.attrs(self)["keydep"][key]
+            attrs["keydep"][key].notify()
+            del attrs["keydep"][key]
         return retval
 
     return inner
@@ -312,9 +330,10 @@ def delete_key_trap(method, obj_cls):
             raise StateModifiedError()
         retval = fn(self.target, *args, **kwargs)
         key = args[0]
-        proxy_db.attrs(self)["dep"].notify()
-        proxy_db.attrs(self)["keydep"][key].notify()
-        del proxy_db.attrs(self)["keydep"][key]
+        attrs = proxy_db.attrs(self)
+        attrs["dep"].notify()
+        attrs["keydep"][key].notify()
+        del attrs["keydep"][key]
         return retval
 
     return inner
